@@ -9,12 +9,20 @@
 
 R_CallMethodDef CallMethods[] = {
   {"mgcv_pmmult2", (DL_FUNC) &mgcv_pmmult2,5},
-  {"mgcv_Rpiqr", (DL_FUNC) &mgcv_Rpiqr,4}, 
-  { "mgcv_tmm",(DL_FUNC)&mgcv_tmm,5},
+  {"mgcv_Rpiqr", (DL_FUNC) &mgcv_Rpiqr,5}, 
+  { "mgcv_tmm",(DL_FUNC)&mgcv_tmm,5}, 
+  { "mgcv_Rpbsi",(DL_FUNC)&mgcv_Rpbsi,2},
+  { "mgcv_RPPt",(DL_FUNC)&mgcv_RPPt,3},
+  { "mgcv_Rpchol",(DL_FUNC)&mgcv_Rpchol,4},
   {NULL, NULL, 0}
 };
 
 R_CMethodDef CEntries[] = {
+    {"mgcv_omp", (DL_FUNC) &mgcv_omp, 1},
+    {"coxpred", (DL_FUNC) &coxpred, 13},
+    {"coxpp", (DL_FUNC) &coxpp, 10},
+    {"coxlpl", (DL_FUNC) &coxlpl, 17},
+    {"mvn_ll", (DL_FUNC) &mvn_ll,15},
     {"RMonoCon", (DL_FUNC) &RMonoCon, 7},
     {"RuniqueCombs", (DL_FUNC) &RuniqueCombs, 4},
     {"RPCLS", (DL_FUNC) &RPCLS, 14},
@@ -25,8 +33,8 @@ R_CMethodDef CEntries[] = {
     {"magic", (DL_FUNC) &magic, 19},
     {"mgcv_mmult", (DL_FUNC) &mgcv_mmult,8},
     {"mgcv_pmmult", (DL_FUNC) &mgcv_pmmult,9},
-    {"gdi1",(DL_FUNC) &gdi1,46},
-    {"gdi2",(DL_FUNC) &gdi2,43},
+    {"gdi1",(DL_FUNC) &gdi1,47},
+    {"gdi2",(DL_FUNC) &gdi2,44},
     {"R_cond",(DL_FUNC) &R_cond,5} ,
     {"pls_fit1",(DL_FUNC)&pls_fit1,12},
     {"tweedious",(DL_FUNC)&tweedious,13},
@@ -39,7 +47,7 @@ R_CMethodDef CEntries[] = {
     {"read_mat",(DL_FUNC)&read_mat,4},
     {"rwMatrix",(DL_FUNC)&rwMatrix,6},
     {"in_out",(DL_FUNC)&in_out,8},
-    {"Rlanczos",(DL_FUNC)&Rlanczos,7},
+    {"Rlanczos",(DL_FUNC)&Rlanczos,8},
     {"rksos",(DL_FUNC)&rksos,3},
     {"gen_tps_poly_powers",(DL_FUNC)&gen_tps_poly_powers,4},
     {"k_nn",(DL_FUNC)&k_nn,8},
